#include<iostream>

#include "HTNBase.hh"

using namespace std;

/** Copy constructor.
 * defautTask is automatically set to an empty Task
 * @param htnb Base to copy
 */
HTNBase::HTNBase(HTNBase const& htnb) {
	map<string, Task*> tmp = htnb.getDatabase();
	for(map<string, Task*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it)
		database[it->first] = new Task(*(it->second));
}

/** Add a task in the databse.
 * If there is already an Task with the same name, overwrites it (without freeing memory!)
 * @param t Task to insert
 */
void HTNBase::add(Task* t) {
	if(t != NULL) database[t->getName()] = t;
	else cerr << "ERROR: trying to add an Task in the HTNBase, the Task pointer is NULL (operation ignored)" << endl;
}

/** Check if a given task exists.
 * @param name Task to look for
 * @return True if task exists, false otherwise
 */
bool HTNBase::existingTask(string const& name) const {
	return database.find(name) != database.end();
}

/** Set belief management method for this HTN base.
 * @param bm Belief management method
 */
void HTNBase::setBeliefMethod(BeliefMethod* bm) {
	this->beliefMethod = bm;
}

/** Return the belief management method for this HTN base.
 * @return Current belief management method
 */
BeliefMethod* HTNBase::getBeliefMethod() const {
	return this->beliefMethod;
}

/** Make a string out of the HTNBase and return it.
 * @return String to represent the HTNBase
 */
string HTNBase::toString() const {
	string s = "***** Start HTNBase Printing *****\n";
	for(map<string,Task*>::const_iterator it=database.begin() ; it!=database.end() ; ++it)
		s += it->second->toString() + "\n";
	s += "***** End HTNBase Printing *****\n";
	return s;
}

/** Assignment operator.
 * @param htnb2copy HTNBase to copy
 * @return Copy of the parameter
 */
HTNBase& HTNBase::operator=(HTNBase const& htnb2copy) {
	// we empty database
	for(map<string, Task*>::iterator itD = database.begin() ; itD!=database.end() ; ++itD) delete itD->second;
	database.clear();
	// we complete database from htnb2copy database
	map<string, Task*> tmp = htnb2copy.getDatabase();
	for(map<string, Task*>::const_iterator it = tmp.begin() ; it!=tmp.end() ; ++it) database[it->first] = new Task(*(it->second));
	return *this;
}

/** Destructor.
 * Deletes all Entities in database
 */
HTNBase::~HTNBase() {
	for(map<string,Task*>::iterator it=database.begin() ; it!=database.end() ; ++it)
		delete it->second;
}
